@@ -1,15 +1,16 @@
 import asyncio
 import contextlib
 import gc
+import io
 import json
-import re
-import sys
 from http.cookies import SimpleCookie
-from io import BytesIO
+from typing import Any, List
 from unittest import mock
+from uuid import uuid4
 
 import pytest
 from multidict import CIMultiDict, MultiDict
+from re_assert import Matches
 from yarl import URL
 
 import aiohttp
@@ -17,7 +18,7 @@ from aiohttp import client, hdrs, web
 from aiohttp.client import ClientSession
 from aiohttp.client_reqrep import ClientRequest
 from aiohttp.connector import BaseConnector, TCPConnector
-from aiohttp.helpers import DEBUG, PY_36
+from aiohttp.helpers import DEBUG
 from aiohttp.test_utils import make_mocked_coro
 
 
@@ -30,7 +31,7 @@ def connector(loop):
     proto = mock.Mock()
     conn._conns["a"] = [(proto, 123)]
     yield conn
-    conn.close()
+    loop.run_until_complete(conn.close())
 
 
 @pytest.fixture
@@ -159,7 +160,6 @@ async def test_merge_headers_with_list_of_tuples_duplicated_names(
 
 
 def test_http_GET(session, params) -> None:
-    # Python 3.8 will auto use mock.AsyncMock, it has different behavior
     with mock.patch(
         "aiohttp.client.ClientSession._request", new_callable=mock.MagicMock
     ) as patched:
@@ -292,7 +292,7 @@ async def test_connector(create_session, loop, mocker) -> None:
 
     await session.close()
     assert connector.close.called
-    connector.close()
+    await connector.close()
 
 
 async def test_create_connector(create_session, loop, mocker) -> None:
@@ -314,19 +314,20 @@ def test_connector_loop(loop) -> None:
 
         connector = another_loop.run_until_complete(make_connector())
 
-        stack.enter_context(contextlib.closing(connector))
         with pytest.raises(RuntimeError) as ctx:
 
             async def make_sess():
                 return ClientSession(connector=connector, loop=loop)
 
             loop.run_until_complete(make_sess())
-        assert re.match(
-            "Session and connector has to use same event loop", str(ctx.value)
+        assert (
+            Matches("Session and connector has to use same event loop")
+            == str(ctx.value).strip()
         )
+        another_loop.run_until_complete(connector.close())
 
 
-def test_detach(session) -> None:
+def test_detach(loop, session) -> None:
     conn = session.connector
     try:
         assert not conn.closed
@@ -335,7 +336,7 @@ def test_detach(session) -> None:
         assert session.closed
         assert not conn.closed
     finally:
-        conn.close()
+        loop.run_until_complete(conn.close())
 
 
 async def test_request_closed_session(session) -> None:
@@ -344,10 +345,10 @@ async def test_request_closed_session(session) -> None:
         await session.request("get", "/")
 
 
-def test_close_flag_for_closed_connector(session) -> None:
+def test_close_flag_for_closed_connector(loop, session) -> None:
     conn = session.connector
     assert not session.closed
-    conn.close()
+    loop.run_until_complete(conn.close())
     assert session.closed
 
 
@@ -397,7 +398,7 @@ async def test_del_debug(connector, loop) -> None:
     assert logs[0] == expected
 
 
-async def test_context_manager(connector, loop) -> None:
+async def test_session_context_manager(connector, loop) -> None:
     with pytest.raises(TypeError):
         with ClientSession(loop=loop, connector=connector) as session:
             pass
@@ -471,7 +472,61 @@ async def test_close_conn_on_error(create_session) -> None:
         c.__del__()
 
 
-async def test_cookie_jar_usage(loop, aiohttp_client) -> None:
+@pytest.mark.parametrize("protocol", ["http", "https", "ws", "wss"])
+async def test_ws_connect_allowed_protocols(
+    create_session: Any,
+    create_mocked_conn: Any,
+    protocol: str,
+    ws_key: Any,
+    key_data: Any,
+) -> None:
+    resp = mock.create_autospec(aiohttp.ClientResponse)
+    resp.status = 101
+    resp.headers = {
+        hdrs.UPGRADE: "websocket",
+        hdrs.CONNECTION: "upgrade",
+        hdrs.SEC_WEBSOCKET_ACCEPT: ws_key,
+    }
+    resp.url = URL(f"{protocol}://example.com")
+    resp.cookies = SimpleCookie()
+    resp.start = mock.AsyncMock()
+
+    req = mock.create_autospec(aiohttp.ClientRequest, spec_set=True)
+    req_factory = mock.Mock(return_value=req)
+    req.send = mock.AsyncMock(return_value=resp)
+
+    session = await create_session(request_class=req_factory)
+
+    connections = []
+    original_connect = session._connector.connect
+
+    async def connect(req, traces, timeout):
+        conn = await original_connect(req, traces, timeout)
+        connections.append(conn)
+        return conn
+
+    async def create_connection(req, traces, timeout):
+        return create_mocked_conn()
+
+    connector = session._connector
+    with mock.patch.object(connector, "connect", connect), mock.patch.object(
+        connector, "_create_connection", create_connection
+    ), mock.patch.object(connector, "_release"), mock.patch(
+        "aiohttp.client.os"
+    ) as m_os:
+        m_os.urandom.return_value = key_data
+        await session.ws_connect(f"{protocol}://example.com")
+
+    # normally called during garbage collection.  triggers an exception
+    # if the connection wasn't already closed
+    for c in connections:
+        c.close()
+        c.__del__()
+
+    await session.close()
+
+
+async def test_cookie_jar_usage(loop: Any, aiohttp_client: Any) -> None:
     req_url = None
 
     jar = mock.Mock()
@@ -513,6 +568,7 @@ async def test_cookie_jar_usage(loop, aiohttp_client) -> None:
 async def test_session_default_version(loop) -> None:
     session = aiohttp.ClientSession(loop=loop)
     assert session.version == aiohttp.HttpVersion11
+    await session.close()
 
 
 async def test_session_loop(loop) -> None:
@@ -547,53 +603,167 @@ async def test_request_tracing(loop, aiohttp_client) -> None:
     trace_config_ctx = mock.Mock()
     trace_request_ctx = {}
     body = "This is request body"
-    gathered_req_body = BytesIO()
-    gathered_res_body = BytesIO()
+    gathered_req_headers = CIMultiDict()
     on_request_start = mock.Mock(side_effect=make_mocked_coro(mock.Mock()))
     on_request_redirect = mock.Mock(side_effect=make_mocked_coro(mock.Mock()))
     on_request_end = mock.Mock(side_effect=make_mocked_coro(mock.Mock()))
 
-    async def on_request_chunk_sent(session, context, params):
-        gathered_req_body.write(params.chunk)
+    with io.BytesIO() as gathered_req_body, io.BytesIO() as gathered_res_body:
 
-    async def on_response_chunk_received(session, context, params):
-        gathered_res_body.write(params.chunk)
+        async def on_request_chunk_sent(session, context, params):
+            gathered_req_body.write(params.chunk)
+
+        async def on_response_chunk_received(session, context, params):
+            gathered_res_body.write(params.chunk)
+
+        async def on_request_headers_sent(session, context, params):
+            gathered_req_headers.extend(**params.headers)
+
+        trace_config = aiohttp.TraceConfig(
+            trace_config_ctx_factory=mock.Mock(return_value=trace_config_ctx)
+        )
+        trace_config.on_request_start.append(on_request_start)
+        trace_config.on_request_end.append(on_request_end)
+        trace_config.on_request_chunk_sent.append(on_request_chunk_sent)
+        trace_config.on_response_chunk_received.append(on_response_chunk_received)
+        trace_config.on_request_redirect.append(on_request_redirect)
+        trace_config.on_request_headers_sent.append(on_request_headers_sent)
+
+        headers = CIMultiDict({"Custom-Header": "Custom value"})
+        session = await aiohttp_client(
+            app, trace_configs=[trace_config], headers=headers
+        )
+
+        async with session.post(
+            "/", data=body, trace_request_ctx=trace_request_ctx
+        ) as resp:
+
+            await resp.json()
+
+            on_request_start.assert_called_once_with(
+                session.session,
+                trace_config_ctx,
+                aiohttp.TraceRequestStartParams(
+                    hdrs.METH_POST, session.make_url("/"), headers
+                ),
+            )
+
+            on_request_end.assert_called_once_with(
+                session.session,
+                trace_config_ctx,
+                aiohttp.TraceRequestEndParams(
+                    hdrs.METH_POST, session.make_url("/"), headers, resp
+                ),
+            )
+            assert not on_request_redirect.called
+            assert gathered_req_body.getvalue() == body.encode("utf8")
+            assert gathered_res_body.getvalue() == json.dumps({"ok": True}).encode(
+                "utf8"
+            )
+            assert gathered_req_headers["Custom-Header"] == "Custom value"
+
+
+async def test_request_tracing_url_params(loop: Any, aiohttp_client: Any) -> None:
+    async def root_handler(request):
+        return web.Response()
+
+    async def redirect_handler(request):
+        raise web.HTTPFound("/")
+
+    app = web.Application()
+    app.router.add_get("/", root_handler)
+    app.router.add_get("/redirect", redirect_handler)
+
+    mocks = [mock.Mock(side_effect=make_mocked_coro(mock.Mock())) for _ in range(7)]
+    (
+        on_request_start,
+        on_request_redirect,
+        on_request_end,
+        on_request_exception,
+        on_request_chunk_sent,
+        on_response_chunk_received,
+        on_request_headers_sent,
+    ) = mocks
 
     trace_config = aiohttp.TraceConfig(
-        trace_config_ctx_factory=mock.Mock(return_value=trace_config_ctx)
+        trace_config_ctx_factory=mock.Mock(return_value=mock.Mock())
     )
     trace_config.on_request_start.append(on_request_start)
+    trace_config.on_request_redirect.append(on_request_redirect)
     trace_config.on_request_end.append(on_request_end)
+    trace_config.on_request_exception.append(on_request_exception)
     trace_config.on_request_chunk_sent.append(on_request_chunk_sent)
     trace_config.on_response_chunk_received.append(on_response_chunk_received)
-    trace_config.on_request_redirect.append(on_request_redirect)
+    trace_config.on_request_headers_sent.append(on_request_headers_sent)
 
     session = await aiohttp_client(app, trace_configs=[trace_config])
 
-    async with session.post(
-        "/", data=body, trace_request_ctx=trace_request_ctx
-    ) as resp:
+    def reset_mocks() -> None:
+        for m in mocks:
+            m.reset_mock()
 
-        await resp.json()
+    def to_trace_urls(mock_func: mock.Mock) -> List[URL]:
+        return [call_args[0][-1].url for call_args in mock_func.call_args_list]
 
-        on_request_start.assert_called_once_with(
-            session.session,
-            trace_config_ctx,
-            aiohttp.TraceRequestStartParams(
-                hdrs.METH_POST, session.make_url("/"), CIMultiDict()
-            ),
-        )
+    def to_url(path: str) -> URL:
+        return session.make_url(path)
 
-        on_request_end.assert_called_once_with(
-            session.session,
-            trace_config_ctx,
-            aiohttp.TraceRequestEndParams(
-                hdrs.METH_POST, session.make_url("/"), CIMultiDict(), resp
-            ),
-        )
-        assert not on_request_redirect.called
-        assert gathered_req_body.getvalue() == body.encode("utf8")
-        assert gathered_res_body.getvalue() == json.dumps({"ok": True}).encode("utf8")
+    # Standard
+    for req in [
+        lambda: session.get("/?x=0"),
+        lambda: session.get("/", params=dict(x=0)),
+    ]:
+        reset_mocks()
+        async with req() as resp:
+            await resp.text()
+            assert to_trace_urls(on_request_start) == [to_url("/?x=0")]
+            assert to_trace_urls(on_request_redirect) == []
+            assert to_trace_urls(on_request_end) == [to_url("/?x=0")]
+            assert to_trace_urls(on_request_exception) == []
+            assert to_trace_urls(on_request_chunk_sent) == [to_url("/?x=0")]
+            assert to_trace_urls(on_response_chunk_received) == [to_url("/?x=0")]
+            assert to_trace_urls(on_request_headers_sent) == [to_url("/?x=0")]
+
+    # Redirect
+    for req in [
+        lambda: session.get("/redirect?x=0"),
+        lambda: session.get("/redirect", params=dict(x=0)),
+    ]:
+        reset_mocks()
+        async with req() as resp:
+            await resp.text()
+            assert to_trace_urls(on_request_start) == [to_url("/redirect?x=0")]
+            assert to_trace_urls(on_request_redirect) == [to_url("/redirect?x=0")]
+            assert to_trace_urls(on_request_end) == [to_url("/")]
+            assert to_trace_urls(on_request_exception) == []
+            assert to_trace_urls(on_request_chunk_sent) == [
+                to_url("/redirect?x=0"),
+                to_url("/"),
+            ]
+            assert to_trace_urls(on_response_chunk_received) == [to_url("/")]
+            assert to_trace_urls(on_request_headers_sent) == [
+                to_url("/redirect?x=0"),
+                to_url("/"),
+            ]
+
+    # Exception
+    with mock.patch("aiohttp.client.TCPConnector.connect") as connect_patched:
+        connect_patched.side_effect = Exception()
+
+        for req in [
+            lambda: session.get("/?x=0"),
+            lambda: session.get("/", params=dict(x=0)),
+        ]:
+            reset_mocks()
+            with contextlib.suppress(Exception):
+                await req()
+            assert to_trace_urls(on_request_start) == [to_url("/?x=0")]
+            assert to_trace_urls(on_request_redirect) == []
+            assert to_trace_urls(on_request_end) == []
+            assert to_trace_urls(on_request_exception) == [to_url("?x=0")]
+            assert to_trace_urls(on_request_chunk_sent) == []
+            assert to_trace_urls(on_response_chunk_received) == []
+            assert to_trace_urls(on_request_headers_sent) == []
 
 
 async def test_request_tracing_exception() -> None:
@@ -607,13 +777,7 @@ async def test_request_tracing_exception() -> None:
 
     with mock.patch("aiohttp.client.TCPConnector.connect") as connect_patched:
         error = Exception()
-        if sys.version_info >= (3, 8, 1):
-            connect_patched.side_effect = error
-        else:
-            loop = asyncio.get_event_loop()
-            f = loop.create_future()
-            f.set_exception(error)
-            connect_patched.return_value = f
+        connect_patched.side_effect = error
 
         session = aiohttp.ClientSession(loop=loop, trace_configs=[trace_config])
 
@@ -631,6 +795,8 @@ async def test_request_tracing_exception() -> None:
         )
         assert not on_request_end.called
 
+    await session.close()
+
 
 async def test_request_tracing_interpose_headers(loop, aiohttp_client) -> None:
     async def handler(request):
@@ -643,7 +809,7 @@ async def test_request_tracing_interpose_headers(loop, aiohttp_client) -> None:
         headers = None
 
         def __init__(self, *args, **kwargs):
-            super(MyClientRequest, self).__init__(*args, **kwargs)
+            super().__init__(*args, **kwargs)
             MyClientRequest.headers = self.headers
 
     async def new_headers(session, trace_config_ctx, data):
@@ -660,7 +826,6 @@ async def test_request_tracing_interpose_headers(loop, aiohttp_client) -> None:
     assert MyClientRequest.headers["foo"] == "bar"
 
 
-@pytest.mark.skipif(not PY_36, reason="Python 3.6+ required")
 def test_client_session_inheritance() -> None:
     with pytest.warns(DeprecationWarning):
 
@@ -673,6 +838,7 @@ async def test_client_session_custom_attr(loop) -> None:
     session = ClientSession(loop=loop)
     with pytest.warns(DeprecationWarning):
         session.custom = None
+    await session.close()
 
 
 async def test_client_session_timeout_args(loop) -> None:
@@ -693,25 +859,42 @@ async def test_client_session_timeout_args(loop) -> None:
             loop=loop, timeout=client.ClientTimeout(total=10 * 60), conn_timeout=30 * 60
         )
 
+    await session1.close()
+    await session2.close()
+
 
 async def test_client_session_timeout_default_args(loop) -> None:
     session1 = ClientSession()
     assert session1.timeout == client.DEFAULT_TIMEOUT
+    await session1.close()
 
 
-async def test_client_session_timeout_argument() -> None:
-    session = ClientSession(timeout=500)
-    assert session.timeout == 500
+async def test_client_session_timeout_zero() -> None:
+    timeout = client.ClientTimeout(total=10, connect=0, sock_connect=0, sock_read=0)
+    try:
+        async with ClientSession(timeout=timeout) as session:
+            await session.get("http://example.com")
+    except asyncio.TimeoutError:
+        pytest.fail("0 should disable timeout.")
+
+
+async def test_client_session_timeout_bad_argument() -> None:
+    with pytest.raises(ValueError):
+        ClientSession(timeout="test_bad_argumnet")
+    with pytest.raises(ValueError):
+        ClientSession(timeout=100)
 
 
 async def test_requote_redirect_url_default() -> None:
     session = ClientSession()
     assert session.requote_redirect_url
+    await session.close()
 
 
 async def test_requote_redirect_url_default_disable() -> None:
     session = ClientSession(requote_redirect_url=False)
     assert not session.requote_redirect_url
+    await session.close()
 
 
 async def test_requote_redirect_setter() -> None:
@@ -720,3 +903,70 @@ async def test_requote_redirect_setter() -> None:
     with pytest.warns(DeprecationWarning):
         session.requote_redirect_url = False
     assert not session.requote_redirect_url
+    await session.close()
+
+
+@pytest.mark.parametrize(
+    ("base_url", "url", "expected_url"),
+    [
+        pytest.param(
+            None,
+            "http://example.com/test",
+            URL("http://example.com/test"),
+            id="base_url=None url='http://example.com/test'",
+        ),
+        pytest.param(
+            None,
+            URL("http://example.com/test"),
+            URL("http://example.com/test"),
+            id="base_url=None url=URL('http://example.com/test')",
+        ),
+        pytest.param(
+            "http://example.com",
+            "/test",
+            URL("http://example.com/test"),
+            id="base_url='http://example.com' url='/test'",
+        ),
+        pytest.param(
+            URL("http://example.com"),
+            "/test",
+            URL("http://example.com/test"),
+            id="base_url=URL('http://example.com') url='/test'",
+        ),
+    ],
+)
+async def test_build_url_returns_expected_url(
+    create_session, base_url, url, expected_url
+) -> None:
+    session = await create_session(base_url)
+    assert session._build_url(url) == expected_url
+
+
+async def test_instantiation_with_invalid_timeout_value(loop):
+    loop.set_debug(False)
+    logs = []
+    loop.set_exception_handler(lambda loop, ctx: logs.append(ctx))
+    with pytest.raises(ValueError, match="timeout parameter cannot be .*"):
+        ClientSession(timeout=1)
+    # should not have "Unclosed client session" warning
+    assert not logs
+
+
+@pytest.mark.parametrize(
+    ("outer_name", "inner_name"),
+    [
+        ("skip_auto_headers", "_skip_auto_headers"),
+        ("auth", "_default_auth"),
+        ("json_serialize", "_json_serialize"),
+        ("connector_owner", "_connector_owner"),
+        ("raise_for_status", "_raise_for_status"),
+        ("trust_env", "_trust_env"),
+        ("trace_configs", "_trace_configs"),
+    ],
+)
+async def test_properties(
+    session: ClientSession, outer_name: str, inner_name: str
+) -> None:
+    value = uuid4()
+    setattr(session, inner_name, value)
+    assert value == getattr(session, outer_name)
