@@ -2,6 +2,7 @@ import asyncio
 import base64
 import hashlib
 import os
+from typing import Any
 from unittest import mock
 
 import pytest
@@ -13,30 +14,16 @@ from aiohttp.streams import EofStream
 from aiohttp.test_utils import make_mocked_coro
 
 
-@pytest.fixture
-def key_data():
-    return os.urandom(16)
-
-
-@pytest.fixture
-def key(key_data):
-    return base64.b64encode(key_data)
-
-
-@pytest.fixture
-def ws_key(key):
-    return base64.b64encode(hashlib.sha1(key + WS_KEY).digest()).decode()
-
-
-async def test_ws_connect(ws_key, loop, key_data) -> None:
+async def test_ws_connect(ws_key: Any, loop: Any, key_data: Any) -> None:
     resp = mock.Mock()
     resp.status = 101
     resp.headers = {
-        hdrs.UPGRADE: hdrs.WEBSOCKET,
-        hdrs.CONNECTION: hdrs.UPGRADE,
+        hdrs.UPGRADE: "websocket",
+        hdrs.CONNECTION: "upgrade",
         hdrs.SEC_WEBSOCKET_ACCEPT: ws_key,
         hdrs.SEC_WEBSOCKET_PROTOCOL: "chat",
     }
+    resp.connection.protocol.read_timeout = None
     with mock.patch("aiohttp.client.os") as m_os:
         with mock.patch("aiohttp.client.ClientSession.request") as m_req:
             m_os.urandom.return_value = key_data
@@ -52,6 +39,97 @@ async def test_ws_connect(ws_key, loop, key_data) -> None:
     assert hdrs.ORIGIN not in m_req.call_args[1]["headers"]
 
 
+async def test_ws_connect_read_timeout_is_reset_to_inf(
+    ws_key: Any, loop: Any, key_data: Any
+) -> None:
+    resp = mock.Mock()
+    resp.status = 101
+    resp.headers = {
+        hdrs.UPGRADE: "websocket",
+        hdrs.CONNECTION: "upgrade",
+        hdrs.SEC_WEBSOCKET_ACCEPT: ws_key,
+        hdrs.SEC_WEBSOCKET_PROTOCOL: "chat",
+    }
+    resp.connection.protocol.read_timeout = 0.5
+    with mock.patch("aiohttp.client.os") as m_os, mock.patch(
+        "aiohttp.client.ClientSession.request"
+    ) as m_req:
+        m_os.urandom.return_value = key_data
+        m_req.return_value = loop.create_future()
+        m_req.return_value.set_result(resp)
+
+        res = await aiohttp.ClientSession().ws_connect(
+            "http://test.org", protocols=("t1", "t2", "chat")
+        )
+
+    assert isinstance(res, client.ClientWebSocketResponse)
+    assert res.protocol == "chat"
+    assert hdrs.ORIGIN not in m_req.call_args[1]["headers"]
+    assert resp.connection.protocol.read_timeout is None
+
+
+async def test_ws_connect_read_timeout_stays_inf(
+    ws_key: Any, loop: Any, key_data: Any
+) -> None:
+    resp = mock.Mock()
+    resp.status = 101
+    resp.headers = {
+        hdrs.UPGRADE: "websocket",
+        hdrs.CONNECTION: "upgrade",
+        hdrs.SEC_WEBSOCKET_ACCEPT: ws_key,
+        hdrs.SEC_WEBSOCKET_PROTOCOL: "chat",
+    }
+    resp.connection.protocol.read_timeout = None
+    with mock.patch("aiohttp.client.os") as m_os, mock.patch(
+        "aiohttp.client.ClientSession.request"
+    ) as m_req:
+        m_os.urandom.return_value = key_data
+        m_req.return_value = loop.create_future()
+        m_req.return_value.set_result(resp)
+
+        res = await aiohttp.ClientSession().ws_connect(
+            "http://test.org",
+            protocols=("t1", "t2", "chat"),
+            receive_timeout=0.5,
+        )
+
+    assert isinstance(res, client.ClientWebSocketResponse)
+    assert res.protocol == "chat"
+    assert hdrs.ORIGIN not in m_req.call_args[1]["headers"]
+    assert resp.connection.protocol.read_timeout is None
+
+
+async def test_ws_connect_read_timeout_reset_to_max(
+    ws_key: Any, loop: Any, key_data: Any
+) -> None:
+    resp = mock.Mock()
+    resp.status = 101
+    resp.headers = {
+        hdrs.UPGRADE: "websocket",
+        hdrs.CONNECTION: "upgrade",
+        hdrs.SEC_WEBSOCKET_ACCEPT: ws_key,
+        hdrs.SEC_WEBSOCKET_PROTOCOL: "chat",
+    }
+    resp.connection.protocol.read_timeout = 0.5
+    with mock.patch("aiohttp.client.os") as m_os, mock.patch(
+        "aiohttp.client.ClientSession.request"
+    ) as m_req:
+        m_os.urandom.return_value = key_data
+        m_req.return_value = loop.create_future()
+        m_req.return_value.set_result(resp)
+
+        res = await aiohttp.ClientSession().ws_connect(
+            "http://test.org",
+            protocols=("t1", "t2", "chat"),
+            receive_timeout=1.0,
+        )
+
+    assert isinstance(res, client.ClientWebSocketResponse)
+    assert res.protocol == "chat"
+    assert hdrs.ORIGIN not in m_req.call_args[1]["headers"]
+    assert resp.connection.protocol.read_timeout == 1.0
+
+
 async def test_ws_connect_with_origin(key_data, loop) -> None:
     resp = mock.Mock()
     resp.status = 403
@@ -71,6 +149,31 @@ async def test_ws_connect_with_origin(key_data, loop) -> None:
     assert m_req.call_args[1]["headers"][hdrs.ORIGIN] == origin
 
 
+async def test_ws_connect_with_params(ws_key, loop, key_data) -> None:
+    params = {"key1": "value1", "key2": "value2"}
+
+    resp = mock.Mock()
+    resp.status = 101
+    resp.headers = {
+        hdrs.UPGRADE: "websocket",
+        hdrs.CONNECTION: "upgrade",
+        hdrs.SEC_WEBSOCKET_ACCEPT: ws_key,
+        hdrs.SEC_WEBSOCKET_PROTOCOL: "chat",
+    }
+    resp.connection.protocol.read_timeout = None
+    with mock.patch("aiohttp.client.os") as m_os:
+        with mock.patch("aiohttp.client.ClientSession.request") as m_req:
+            m_os.urandom.return_value = key_data
+            m_req.return_value = loop.create_future()
+            m_req.return_value.set_result(resp)
+
+            await aiohttp.ClientSession().ws_connect(
+                "http://test.org", protocols=("t1", "t2", "chat"), params=params
+            )
+
+    assert m_req.call_args[1]["params"] == params
+
+
 async def test_ws_connect_custom_response(loop, ws_key, key_data) -> None:
     class CustomResponse(client.ClientWebSocketResponse):
         def read(self, decode=False):
@@ -79,10 +182,11 @@ async def test_ws_connect_custom_response(loop, ws_key, key_data) -> None:
     resp = mock.Mock()
     resp.status = 101
     resp.headers = {
-        hdrs.UPGRADE: hdrs.WEBSOCKET,
-        hdrs.CONNECTION: hdrs.UPGRADE,
+        hdrs.UPGRADE: "websocket",
+        hdrs.CONNECTION: "upgrade",
         hdrs.SEC_WEBSOCKET_ACCEPT: ws_key,
     }
+    resp.connection.protocol.read_timeout = None
     with mock.patch("aiohttp.client.os") as m_os:
         with mock.patch("aiohttp.client.ClientSession.request") as m_req:
             m_os.urandom.return_value = key_data
@@ -100,8 +204,8 @@ async def test_ws_connect_err_status(loop, ws_key, key_data) -> None:
     resp = mock.Mock()
     resp.status = 500
     resp.headers = {
-        hdrs.UPGRADE: hdrs.WEBSOCKET,
-        hdrs.CONNECTION: hdrs.UPGRADE,
+        hdrs.UPGRADE: "websocket",
+        hdrs.CONNECTION: "upgrade",
         hdrs.SEC_WEBSOCKET_ACCEPT: ws_key,
     }
     with mock.patch("aiohttp.client.os") as m_os:
@@ -123,7 +227,7 @@ async def test_ws_connect_err_upgrade(loop, ws_key, key_data) -> None:
     resp.status = 101
     resp.headers = {
         hdrs.UPGRADE: "test",
-        hdrs.CONNECTION: hdrs.UPGRADE,
+        hdrs.CONNECTION: "upgrade",
         hdrs.SEC_WEBSOCKET_ACCEPT: ws_key,
     }
     with mock.patch("aiohttp.client.os") as m_os:
@@ -144,7 +248,7 @@ async def test_ws_connect_err_conn(loop, ws_key, key_data) -> None:
     resp = mock.Mock()
     resp.status = 101
     resp.headers = {
-        hdrs.UPGRADE: hdrs.WEBSOCKET,
+        hdrs.UPGRADE: "websocket",
         hdrs.CONNECTION: "close",
         hdrs.SEC_WEBSOCKET_ACCEPT: ws_key,
     }
@@ -166,8 +270,8 @@ async def test_ws_connect_err_challenge(loop, ws_key, key_data) -> None:
     resp = mock.Mock()
     resp.status = 101
     resp.headers = {
-        hdrs.UPGRADE: hdrs.WEBSOCKET,
-        hdrs.CONNECTION: hdrs.UPGRADE,
+        hdrs.UPGRADE: "websocket",
+        hdrs.CONNECTION: "upgrade",
         hdrs.SEC_WEBSOCKET_ACCEPT: "asdfasdfasdfasdfasdfasdf",
     }
     with mock.patch("aiohttp.client.os") as m_os:
@@ -200,11 +304,12 @@ async def test_ws_connect_common_headers(ws_key, loop, key_data) -> None:
                 hashlib.sha1(base64.b64encode(base64.b64decode(key)) + WS_KEY).digest()
             ).decode()
             resp.headers = {
-                hdrs.UPGRADE: hdrs.WEBSOCKET,
-                hdrs.CONNECTION: hdrs.UPGRADE,
+                hdrs.UPGRADE: "websocket",
+                hdrs.CONNECTION: "upgrade",
                 hdrs.SEC_WEBSOCKET_ACCEPT: accept,
                 hdrs.SEC_WEBSOCKET_PROTOCOL: "chat",
             }
+            resp.connection.protocol.read_timeout = None
             return resp
 
         with mock.patch("aiohttp.client.os") as m_os:
@@ -231,10 +336,11 @@ async def test_close(loop, ws_key, key_data) -> None:
     resp = mock.Mock()
     resp.status = 101
     resp.headers = {
-        hdrs.UPGRADE: hdrs.WEBSOCKET,
-        hdrs.CONNECTION: hdrs.UPGRADE,
+        hdrs.UPGRADE: "websocket",
+        hdrs.CONNECTION: "upgrade",
         hdrs.SEC_WEBSOCKET_ACCEPT: ws_key,
     }
+    resp.connection.protocol.read_timeout = None
     with mock.patch("aiohttp.client.WebSocketWriter") as WebSocketWriter:
         with mock.patch("aiohttp.client.os") as m_os:
             with mock.patch("aiohttp.client.ClientSession.request") as m_req:
@@ -271,10 +377,11 @@ async def test_close_eofstream(loop, ws_key, key_data) -> None:
     resp = mock.Mock()
     resp.status = 101
     resp.headers = {
-        hdrs.UPGRADE: hdrs.WEBSOCKET,
-        hdrs.CONNECTION: hdrs.UPGRADE,
+        hdrs.UPGRADE: "websocket",
+        hdrs.CONNECTION: "upgrade",
         hdrs.SEC_WEBSOCKET_ACCEPT: ws_key,
     }
+    resp.connection.protocol.read_timeout = None
     with mock.patch("aiohttp.client.WebSocketWriter") as WebSocketWriter:
         with mock.patch("aiohttp.client.os") as m_os:
             with mock.patch("aiohttp.client.ClientSession.request") as m_req:
@@ -301,10 +408,11 @@ async def test_close_exc(loop, ws_key, key_data) -> None:
     resp = mock.Mock()
     resp.status = 101
     resp.headers = {
-        hdrs.UPGRADE: hdrs.WEBSOCKET,
-        hdrs.CONNECTION: hdrs.UPGRADE,
+        hdrs.UPGRADE: "websocket",
+        hdrs.CONNECTION: "upgrade",
         hdrs.SEC_WEBSOCKET_ACCEPT: ws_key,
     }
+    resp.connection.protocol.read_timeout = None
     with mock.patch("aiohttp.client.WebSocketWriter") as WebSocketWriter:
         with mock.patch("aiohttp.client.os") as m_os:
             with mock.patch("aiohttp.client.ClientSession.request") as m_req:
@@ -333,10 +441,11 @@ async def test_close_exc2(loop, ws_key, key_data) -> None:
     resp = mock.Mock()
     resp.status = 101
     resp.headers = {
-        hdrs.UPGRADE: hdrs.WEBSOCKET,
-        hdrs.CONNECTION: hdrs.UPGRADE,
+        hdrs.UPGRADE: "websocket",
+        hdrs.CONNECTION: "upgrade",
         hdrs.SEC_WEBSOCKET_ACCEPT: ws_key,
     }
+    resp.connection.protocol.read_timeout = None
     with mock.patch("aiohttp.client.WebSocketWriter") as WebSocketWriter:
         with mock.patch("aiohttp.client.os") as m_os:
             with mock.patch("aiohttp.client.ClientSession.request") as m_req:
@@ -367,10 +476,11 @@ async def test_send_data_after_close(ws_key, key_data, loop) -> None:
     resp = mock.Mock()
     resp.status = 101
     resp.headers = {
-        hdrs.UPGRADE: hdrs.WEBSOCKET,
-        hdrs.CONNECTION: hdrs.UPGRADE,
+        hdrs.UPGRADE: "websocket",
+        hdrs.CONNECTION: "upgrade",
         hdrs.SEC_WEBSOCKET_ACCEPT: ws_key,
     }
+    resp.connection.protocol.read_timeout = None
     with mock.patch("aiohttp.client.os") as m_os:
         with mock.patch("aiohttp.client.ClientSession.request") as m_req:
             m_os.urandom.return_value = key_data
@@ -395,10 +505,11 @@ async def test_send_data_type_errors(ws_key, key_data, loop) -> None:
     resp = mock.Mock()
     resp.status = 101
     resp.headers = {
-        hdrs.UPGRADE: hdrs.WEBSOCKET,
-        hdrs.CONNECTION: hdrs.UPGRADE,
+        hdrs.UPGRADE: "websocket",
+        hdrs.CONNECTION: "upgrade",
         hdrs.SEC_WEBSOCKET_ACCEPT: ws_key,
     }
+    resp.connection.protocol.read_timeout = None
     with mock.patch("aiohttp.client.WebSocketWriter") as WebSocketWriter:
         with mock.patch("aiohttp.client.os") as m_os:
             with mock.patch("aiohttp.client.ClientSession.request") as m_req:
@@ -423,10 +534,11 @@ async def test_reader_read_exception(ws_key, key_data, loop) -> None:
     hresp = mock.Mock()
     hresp.status = 101
     hresp.headers = {
-        hdrs.UPGRADE: hdrs.WEBSOCKET,
-        hdrs.CONNECTION: hdrs.UPGRADE,
+        hdrs.UPGRADE: "websocket",
+        hdrs.CONNECTION: "upgrade",
         hdrs.SEC_WEBSOCKET_ACCEPT: ws_key,
     }
+    hresp.connection.protocol.read_timeout = None
     with mock.patch("aiohttp.client.WebSocketWriter") as WebSocketWriter:
         with mock.patch("aiohttp.client.os") as m_os:
             with mock.patch("aiohttp.client.ClientSession.request") as m_req:
@@ -465,8 +577,8 @@ async def test_ws_connect_close_resp_on_err(loop, ws_key, key_data) -> None:
     resp = mock.Mock()
     resp.status = 500
     resp.headers = {
-        hdrs.UPGRADE: hdrs.WEBSOCKET,
-        hdrs.CONNECTION: hdrs.UPGRADE,
+        hdrs.UPGRADE: "websocket",
+        hdrs.CONNECTION: "upgrade",
         hdrs.SEC_WEBSOCKET_ACCEPT: ws_key,
     }
     with mock.patch("aiohttp.client.os") as m_os:
@@ -486,11 +598,12 @@ async def test_ws_connect_non_overlapped_protocols(ws_key, loop, key_data) -> No
     resp = mock.Mock()
     resp.status = 101
     resp.headers = {
-        hdrs.UPGRADE: hdrs.WEBSOCKET,
-        hdrs.CONNECTION: hdrs.UPGRADE,
+        hdrs.UPGRADE: "websocket",
+        hdrs.CONNECTION: "upgrade",
         hdrs.SEC_WEBSOCKET_ACCEPT: ws_key,
         hdrs.SEC_WEBSOCKET_PROTOCOL: "other,another",
     }
+    resp.connection.protocol.read_timeout = None
     with mock.patch("aiohttp.client.os") as m_os:
         with mock.patch("aiohttp.client.ClientSession.request") as m_req:
             m_os.urandom.return_value = key_data
@@ -508,11 +621,12 @@ async def test_ws_connect_non_overlapped_protocols_2(ws_key, loop, key_data) ->
     resp = mock.Mock()
     resp.status = 101
     resp.headers = {
-        hdrs.UPGRADE: hdrs.WEBSOCKET,
-        hdrs.CONNECTION: hdrs.UPGRADE,
+        hdrs.UPGRADE: "websocket",
+        hdrs.CONNECTION: "upgrade",
         hdrs.SEC_WEBSOCKET_ACCEPT: ws_key,
         hdrs.SEC_WEBSOCKET_PROTOCOL: "other,another",
     }
+    resp.connection.protocol.read_timeout = None
     with mock.patch("aiohttp.client.os") as m_os:
         with mock.patch("aiohttp.client.ClientSession.request") as m_req:
             m_os.urandom.return_value = key_data
@@ -532,11 +646,12 @@ async def test_ws_connect_deflate(loop, ws_key, key_data) -> None:
     resp = mock.Mock()
     resp.status = 101
     resp.headers = {
-        hdrs.UPGRADE: hdrs.WEBSOCKET,
-        hdrs.CONNECTION: hdrs.UPGRADE,
+        hdrs.UPGRADE: "websocket",
+        hdrs.CONNECTION: "upgrade",
         hdrs.SEC_WEBSOCKET_ACCEPT: ws_key,
         hdrs.SEC_WEBSOCKET_EXTENSIONS: "permessage-deflate",
     }
+    resp.connection.protocol.read_timeout = None
     with mock.patch("aiohttp.client.os") as m_os:
         with mock.patch("aiohttp.client.ClientSession.request") as m_req:
             m_os.urandom.return_value = key_data
@@ -555,11 +670,12 @@ async def test_ws_connect_deflate_per_message(loop, ws_key, key_data) -> None:
     resp = mock.Mock()
     resp.status = 101
     resp.headers = {
-        hdrs.UPGRADE: hdrs.WEBSOCKET,
-        hdrs.CONNECTION: hdrs.UPGRADE,
+        hdrs.UPGRADE: "websocket",
+        hdrs.CONNECTION: "upgrade",
         hdrs.SEC_WEBSOCKET_ACCEPT: ws_key,
         hdrs.SEC_WEBSOCKET_EXTENSIONS: "permessage-deflate",
     }
+    resp.connection.protocol.read_timeout = None
     with mock.patch("aiohttp.client.WebSocketWriter") as WebSocketWriter:
         with mock.patch("aiohttp.client.os") as m_os:
             with mock.patch("aiohttp.client.ClientSession.request") as m_req:
@@ -588,10 +704,11 @@ async def test_ws_connect_deflate_server_not_support(loop, ws_key, key_data) ->
     resp = mock.Mock()
     resp.status = 101
     resp.headers = {
-        hdrs.UPGRADE: hdrs.WEBSOCKET,
-        hdrs.CONNECTION: hdrs.UPGRADE,
+        hdrs.UPGRADE: "websocket",
+        hdrs.CONNECTION: "upgrade",
         hdrs.SEC_WEBSOCKET_ACCEPT: ws_key,
     }
+    resp.connection.protocol.read_timeout = None
     with mock.patch("aiohttp.client.os") as m_os:
         with mock.patch("aiohttp.client.ClientSession.request") as m_req:
             m_os.urandom.return_value = key_data
@@ -610,12 +727,13 @@ async def test_ws_connect_deflate_notakeover(loop, ws_key, key_data) -> None:
     resp = mock.Mock()
     resp.status = 101
     resp.headers = {
-        hdrs.UPGRADE: hdrs.WEBSOCKET,
-        hdrs.CONNECTION: hdrs.UPGRADE,
+        hdrs.UPGRADE: "websocket",
+        hdrs.CONNECTION: "upgrade",
         hdrs.SEC_WEBSOCKET_ACCEPT: ws_key,
         hdrs.SEC_WEBSOCKET_EXTENSIONS: "permessage-deflate; "
         "client_no_context_takeover",
     }
+    resp.connection.protocol.read_timeout = None
     with mock.patch("aiohttp.client.os") as m_os:
         with mock.patch("aiohttp.client.ClientSession.request") as m_req:
             m_os.urandom.return_value = key_data
@@ -634,12 +752,13 @@ async def test_ws_connect_deflate_client_wbits(loop, ws_key, key_data) -> None:
     resp = mock.Mock()
     resp.status = 101
     resp.headers = {
-        hdrs.UPGRADE: hdrs.WEBSOCKET,
-        hdrs.CONNECTION: hdrs.UPGRADE,
+        hdrs.UPGRADE: "websocket",
+        hdrs.CONNECTION: "upgrade",
         hdrs.SEC_WEBSOCKET_ACCEPT: ws_key,
         hdrs.SEC_WEBSOCKET_EXTENSIONS: "permessage-deflate; "
         "client_max_window_bits=10",
     }
+    resp.connection.protocol.read_timeout = None
     with mock.patch("aiohttp.client.os") as m_os:
         with mock.patch("aiohttp.client.ClientSession.request") as m_req:
             m_os.urandom.return_value = key_data
@@ -658,8 +777,8 @@ async def test_ws_connect_deflate_client_wbits_bad(loop, ws_key, key_data) -> No
     resp = mock.Mock()
     resp.status = 101
     resp.headers = {
-        hdrs.UPGRADE: hdrs.WEBSOCKET,
-        hdrs.CONNECTION: hdrs.UPGRADE,
+        hdrs.UPGRADE: "websocket",
+        hdrs.CONNECTION: "upgrade",
         hdrs.SEC_WEBSOCKET_ACCEPT: ws_key,
         hdrs.SEC_WEBSOCKET_EXTENSIONS: "permessage-deflate; "
         "client_max_window_bits=6",
@@ -680,8 +799,8 @@ async def test_ws_connect_deflate_server_ext_bad(loop, ws_key, key_data) -> None
     resp = mock.Mock()
     resp.status = 101
     resp.headers = {
-        hdrs.UPGRADE: hdrs.WEBSOCKET,
-        hdrs.CONNECTION: hdrs.UPGRADE,
+        hdrs.UPGRADE: "websocket",
+        hdrs.CONNECTION: "upgrade",
         hdrs.SEC_WEBSOCKET_ACCEPT: ws_key,
         hdrs.SEC_WEBSOCKET_EXTENSIONS: "permessage-deflate; bad",
     }
